__version__ = '0.3.0'
__author__ = 'CMU'
__author_email__ = 'awillia2@andrew.cmu.edu'
__key_static_file_resnext__ = 'cmu.resnext-101-kinetics.pth'
__digest_static_file_resnext__ = 'f82e4e519723fc7b2ff3761ea35600bdaf796fb7a4e62ee4c5591da7ffe48326'



import unittest
import os

import pandas as pd
from statsforecast.arima import AutoARIMA

from d3m import runtime, index
from d3m.container import dataset
from d3m.metadata import problem
from d3m.metadata.base import ArgumentType, Context
from d3m.metadata.pipeline import Pipeline, PrimitiveStep

DATASET_LOCATION = "/home/mkowales/datasets/sunspots/"
PROBLEM_PATH = os.path.join(DATASET_LOCATION, "TRAIN", "problem_TRAIN", "problemDoc.json")
TRAIN_DOC_PATH = os.path.join(DATASET_LOCATION, "TRAIN", "dataset_TRAIN", "datasetDoc.json")
TEST_DOC_PATH = os.path.join(DATASET_LOCATION, "TEST", "dataset_TEST", "datasetDoc.json")
TRAIN_DATA_PATH = os.path.join(DATASET_LOCATION, "TRAIN", "dataset_TRAIN", "tables", "learningData.csv")
TEST_DATA_PATH = os.path.join(DATASET_LOCATION, "TEST", "dataset_TEST", "tables", "learningData.csv")

class AutoARIMAWrapperTestCase(unittest.TestCase):

    #hyperparams argument is a list of (name, data) tuples
    def construct_pipeline(self, hyperparams) -> Pipeline:
        # Creating pipeline
        pipeline_description = Pipeline()
        pipeline_description.add_input(name='inputs')

        # Step 0: dataset_to_dataframe
        step_0 = PrimitiveStep(primitive=index.get_primitive('d3m.primitives.data_transformation.dataset_to_dataframe.Common'))
        step_0.add_argument(name='inputs', argument_type=ArgumentType.CONTAINER, data='inputs.0')
        step_0.add_output('produce')
        pipeline_description.add_step(step_0)

        # Step 1: profiler
        # Automatically determine semantic types of columns
        step_1 = PrimitiveStep(primitive=index.get_primitive('d3m.primitives.schema_discovery.profiler.Common'))
        step_1.add_argument(name='inputs', argument_type=ArgumentType.CONTAINER, data='steps.0.produce')
        step_1.add_output('produce')
        pipeline_description.add_step(step_1)

        # Step 2: column parser
        step_2 = PrimitiveStep(primitive=index.get_primitive('d3m.primitives.data_transformation.column_parser.Common'))
        step_2.add_argument(name='inputs', argument_type=ArgumentType.CONTAINER, data='steps.1.produce')
        step_2.add_output('produce')
        step_2.add_hyperparameter(
            name='parse_semantic_types',
            argument_type=ArgumentType.VALUE,
            data=[
                "http://schema.org/Boolean",
                "http://schema.org/Integer",
                "http://schema.org/Float",
                "https://metadata.datadrivendiscovery.org/types/FloatVector"
            ]
        )
        pipeline_description.add_step(step_2)

        # Step 3: extract_columns_by_semantic_types(attributes)
        step_3 = PrimitiveStep(primitive=index.get_primitive('d3m.primitives.data_transformation.extract_columns_by_semantic_types.Common'))
        step_3.add_argument(name='inputs', argument_type=ArgumentType.CONTAINER, data='steps.2.produce')
        step_3.add_output('produce')
        step_3.add_hyperparameter(
            name='semantic_types',
            argument_type=ArgumentType.VALUE,
            data=['https://metadata.datadrivendiscovery.org/types/Attribute'],
        )
        pipeline_description.add_step(step_3)

        # Step 4: extract_columns_by_semantic_types(targets)
        step_4 = PrimitiveStep(primitive=index.get_primitive('d3m.primitives.data_transformation.extract_columns_by_semantic_types.Common'))
        step_4.add_argument(name='inputs', argument_type=ArgumentType.CONTAINER, data='steps.2.produce')
        step_4.add_output('produce')
        step_4.add_hyperparameter(
            name='semantic_types',
            argument_type=ArgumentType.VALUE,
            data=[
                "https://metadata.datadrivendiscovery.org/types/Target",
                "https://metadata.datadrivendiscovery.org/types/TrueTarget",
                "https://metadata.datadrivendiscovery.org/types/SuggestedTarget"
            ]
        )
        pipeline_description.add_step(step_4)

        # Step 5: autoARIMA
        step_5 = PrimitiveStep(primitive=index.get_primitive('d3m.primitives.time_series_forecasting.arima.AutonBox'))
        step_5.add_argument(name='inputs', argument_type=ArgumentType.CONTAINER, data='steps.3.produce')
        step_5.add_argument(name='outputs', argument_type=ArgumentType.CONTAINER, data='steps.4.produce')
        #add hyperparams from argument
        for h in hyperparams:
            (name, data) = h
            step_5.add_hyperparameter(
                name = name,
                argument_type = ArgumentType.VALUE,
                data = data
            )
        step_5.add_output('produce')
        pipeline_description.add_step(step_5)

        # Step 6: construct_predictions
        # This is a primitive which assures that the output of a standard pipeline has predictions
        # in the correct structure (e.g., there is also a d3mIndex column with index for every row).
        step_6 = PrimitiveStep(primitive=index.get_primitive('d3m.primitives.data_transformation.construct_predictions.Common'))
        step_6.add_argument(name='inputs', argument_type=ArgumentType.CONTAINER, data='steps.5.produce')
        # This is a primitive which uses a non-standard second argument, named "reference".
        step_6.add_argument(name='reference', argument_type=ArgumentType.CONTAINER, data='steps.0.produce')
        step_6.add_output('produce')
        pipeline_description.add_step(step_6)

        # Final output
        pipeline_description.add_output(name='output predictions', data_reference='steps.6.produce')

        # print json for reference
        #print(pipeline_description.to_json())

        return pipeline_description

    def run_pipeline(self, pipeline_description : Pipeline):
        # Loading problem description.
        problem_description = problem.get_problem(PROBLEM_PATH)

        # Loading train and test datasets.
        train_dataset = dataset.get_dataset(TRAIN_DOC_PATH)
        test_dataset = dataset.get_dataset(TEST_DOC_PATH)

        # Fitting pipeline on train dataset.
        fitted_pipeline, train_predictions, fit_result = runtime.fit(
            pipeline_description,
            [train_dataset],
            problem_description=problem_description,
            context=Context.TESTING,
        )

        # Any errors from running the pipeline are captured and stored in
        # the result objects (together with any values produced until then and
        # pipeline run information). Here we just want to know if it succeed.
        fit_result.check_success()

        # Producing predictions using the fitted pipeline on test dataset.
        test_predictions, produce_result = runtime.produce(
            fitted_pipeline,
            [test_dataset],
        )
        produce_result.check_success()

        return test_predictions.sunspots
    
    '''
    def test_default_params(self):

        #run simple pipeline with AutoARIMA primitive
        pipeline_description = self.construct_pipeline(hyperparams=[])
        pipeline_predictions = self.run_pipeline(pipeline_description).to_numpy().flatten()

        #run statsforecast AutoARIMA directly
        train = pd.read_csv(TRAIN_DATA_PATH)
        test = pd.read_csv(TEST_DATA_PATH)
        y = train.sunspots.to_numpy().flatten()
        h = test.shape[0] #number of rows in test

        arima = AutoARIMA()
        arima.fit(y)
        direct_predictions = arima.predict(h=h, X=).to_numpy().flatten()

        print("direct:")
        print(direct_predictions)
        print("from pipeline:")
        print(pipeline_predictions)

        assert((direct_predictions == pipeline_predictions).all())
    '''

    def test_exogenous(self):

        #run simple pipeline with AutoARIMA primitive
        pipeline_description = self.construct_pipeline(hyperparams=[])
        pipeline_predictions = self.run_pipeline(pipeline_description).to_numpy().flatten()

        #run statsforecast AutoARIMA directly
        train = pd.read_csv(TRAIN_DATA_PATH)
        test = pd.read_csv(TEST_DATA_PATH)
        y = train.sunspots.to_numpy().flatten()
        X = train.loc[:, list(("sd", "observations"))].to_numpy()
        Xf = test.loc[:, list(("sd", "observations"))].to_numpy()
        h = test.shape[0] #number of rows in test

        arima = AutoARIMA()
        arima.fit(y, X=X)
        direct_predictions = arima.predict(h=h, X=Xf).to_numpy().flatten()

        print("direct:")
        print(direct_predictions)
        print("from pipeline:")
        print(pipeline_predictions)

        assert((direct_predictions == pipeline_predictions).all())

if __name__ == '__main__':
    unittest.main()


import typing
import copy

from d3m import container
from d3m.primitive_interfaces import base
from d3m.primitive_interfaces.supervised_learning import SupervisedLearnerPrimitiveBase
from d3m.metadata import base as metadata_base
from d3m.metadata import hyperparams, params
from d3m.exceptions import MissingValueError, PrimitiveNotFittedError

import pandas as pd

from ray import tune
from neuralforecast.auto import AutoNHITS
from neuralforecast.losses.pytorch import MAE
from ray.tune.search.hyperopt import HyperOptSearch
#from neuralforecast import NeuralForecast
from neuralforecast.tsdataset import TimeSeriesDataset

import autonbox

"""
A wrapper primitive for AutoNHITS from NeuralForecast (https://nixtla.github.io/neuralforecast/models.html#autonhits)
More information on Neural Hierarchical Interpolation for Time Series (NHITS): https://nixtla.github.io/neuralforecast/models.nhits.html
For an intro to NeuralForecast, see https://nixtla.github.io/neuralforecast/examples/installation.html


TODO: Add more information here
"""

#not sure if necessary
#TODO: uncomment if this not being here causes errors
#__all__ = ('AutoNHITSPrimitive',)

Inputs = container.DataFrame
Outputs = container.DataFrame

class Params(params.Params):
    has_training_data: bool
    new_training_data: bool

    training_target: typing.Any
    training_attributes: typing.Any
    nf_train_data: typing.Any
    target_name: str
    exog_names: typing.List
    ngroups: int

    fitted: bool
    model: typing.Any

class Hyperparams(hyperparams.Hyperparams):
    pass

class AutoNHITSPrimitive(SupervisedLearnerPrimitiveBase[Inputs, Outputs, Params, Hyperparams]):

    metadata = metadata_base.PrimitiveMetadata({
        "id": "91c8bd09-cf10-4fde-a471-e092ef3df6b4",
        "version": "0.1.0",
        "name": "neuralforecast.models.AutoNHITS",
        "description": "Wrapper of the AutoNHITS model from the neuralforecast package",
        "python_path": "d3m.primitives.time_series_forecasting.nhits.AutonBox",
        "primitive_family": metadata_base.PrimitiveFamily.TIME_SERIES_FORECASTING,
        "algorithm_types": ["DEEP_NEURAL_NETWORK"],
        'source': {
            'name': autonbox.__author__,
            'uris': ['https://github.com/autonlab/autonbox'],
            'contact': 'mailto:mkowales@andrew.cmu.edu'
        },
        "keywords": ["time series", "forecasting", "deep neural network"],
        "installation": [{
            "type": metadata_base.PrimitiveInstallationType.PIP,
            "package": "autonbox",
            "version": autonbox.__version__
        }]
    })

    def __init__(self, *, hyperparams: Hyperparams) -> None:
        print("calling __init__")

        super().__init__(hyperparams=hyperparams)

        self._has_training_data = False
        self._new_training_data = False

        self._training_target = None
        self._training_attributes = None
        self._nf_train_data = None
        self._target_name = None
        self._exog_names = []
        self._ngroups = 0

        self._fitted = False
        self._model = None

    def get_params(self) -> Params:
        print("calling get_params")
        return Params(
            has_training_data = self._has_training_data,
            new_training_data = self._new_training_data,
            training_target = self._training_target,
            training_attributes = self._training_attributes,
            nf_train_data = self._nf_train_data,
            target_name = self._target_name,
            exog_names = self._exog_names,
            ngroups = self._ngroups,
            fitted = self._fitted,
            model = self._model
        )

    def set_params(self, *, params: Params) -> None:
        print("calling set_params")
        self._has_training_data = params['has_training_data']
        self._new_training_data = params['new_training_data']

        self._training_target = params['training_target']
        self._training_attributes = params['training_attributes']
        self._nf_train_data = params['nf_train_data']
        self._target_name = params['target_name']
        self._exog_names = params['exog_names']
        self._ngroups = params['ngroups']

        self._fitted = params['fitted']
        self._model = params['model']

    #private method
    def _format_data(self, attributes, target=None):
        #transform data from d3m input format to neuralforcast ingest format
        print("formatting data for neuralforecast")

        #extract time column as series
        time_col_indices = attributes.metadata.list_columns_with_semantic_types(
            (
                "https://metadata.datadrivendiscovery.org/types/Time",
            )
        )
        print("timestamp cols: " + str(time_col_indices))
        #TODO: make sure there's only 1 timestamp col
        #TODO: make sure it's valid datetime
        time_col = attributes.iloc[:,time_col_indices[0]]
        #print("time col before conversion to datetime:")
        #print(time_col)
        time_col = pd.to_datetime(time_col)
        #print("time col after conversion to datetime:")
        #print(time_col)

        #extract grouping column as series
        group_col_indices = attributes.metadata.list_columns_with_semantic_types(
            (
                "https://metadata.datadrivendiscovery.org/types/GroupingKey",
                "https://metadata.datadrivendiscovery.org/types/SuggestedGroupingKey"
            )
        )
        print("grouping cols: " + str(group_col_indices))
        #TODO: make sure theres <=1 grouping col
        #TODO: make sure grouping col is valid
        if len(group_col_indices) > 0:
            group_col = attributes.iloc[:,group_col_indices[0]]
        else:
            #data has only 1 time series (no grouping col)
            #however neuralforecast still requires a grouping col
            #create a grouping col putting all rows in same group
            group_col = ['a']*attributes.shape[0]
        #TODO: ensure groups are all same length
        self._ngroups = len(set(group_col))

        #extract names of exogneous variable columns and save them
        attribute_col_indices = attributes.metadata.list_columns_with_semantic_types(
            (
                "https://metadata.datadrivendiscovery.org/types/Attribute",
            )
        )
        print("attribute cols: " + str(attribute_col_indices))
        exog_col_inidices = list(set(attribute_col_indices) - set(group_col_indices + time_col_indices))
        print("exogenous cols: " + str(exog_col_inidices))
        exogenous_colnames = [list(attributes.columns)[i] for i in exog_col_inidices]
        #print("exogenous colnames: " + str(exogenous_colnames))
        self._exog_names = exogenous_colnames

        #construct dataframe formatted to be ingested by neuralforecast
        nf_df = copy.deepcopy(attributes[exogenous_colnames])  #exogenous cols retain name from dataset
        #TODO: check that no exogenous cols are named "ds", "y" or "unique_id"
        nf_df['ds'] = time_col
        nf_df['unique_id'] = group_col

        #add target col if we're given one and save target colname
        if target is not None:
            target_colname = target.columns[0]
            nf_df['y'] = target[target_colname]
            self._target_name = target_colname

        return(nf_df)


    def set_training_data(self, *, inputs: Inputs, outputs: Outputs) -> None:
        print("calling set_training_data")
        #print("Inputs:")
        #print(inputs)
        #print("Outputs:")
        #print(outputs)
        
        #inputs is a dataframe that will be used as exogenous data, excepting time columns
        #outputs is a dataframe containing one column, the time series that we want to predict future values of
        
        #TODO: check that outputs has one column
        #TODO: check that inputs and outputs have same number of rows
        #TODO: check at np.nan and np.inf are not present

        if self._has_training_data:
            self._new_training_data = True

        self._has_training_data = True

        #save data in d3m format
        self._training_attributes = inputs
        self._training_target = outputs

        #save data in neuralforecast format
        self._nf_train_data = self._format_data(inputs, outputs)
        #this method also sets self._target_name and self._exog_names

    def fit(self, *, timeout: float = None, iterations: int = None) -> base.CallResult[None]:
        #in order to fit NHITS, need to know forecasting horizon
        #so fit in the produce method and do nothing here
        print("calling fit, do nothing")
        return base.CallResult(None)
    
    #private method
    def _fit_nf(self, train_ts, h):

        print("Fitting NeuralForecast AutoNHITS")
        print("train:")
        print(self._nf_train_data)
        print("h:" + str(h))
        print("future exog: " + str(self._exog_names))

        nhits_config = {
            "input_size": 3*h,
            "n_pool_kernel_size": tune.choice(
                [3 * [1], 3 * [2], 3 * [4], [8, 4, 1], [16, 8, 1]]
            ),
            "n_freq_downsample": tune.choice(
                [
                    [168, 24, 1],
                    [24, 12, 1],
                    [180, 60, 1],
                    [60, 8, 1],
                    [40, 20, 1],
                    [1, 1, 1],
                ]
            ),
            "learning_rate": tune.loguniform(1e-4, 1e-1),
            "scaler_type" : 'robust',
            "max_steps": 100,  #TODO: change to 1000 after testing
            "batch_size": tune.choice([32, 64, 128, 256]),
            "windows_batch_size": tune.choice([128, 256, 512, 1024]),                                                                       # Initial Learning rate
            "random_seed": 1,                               
            "futr_exog_list" : self._exog_names
        }

        self._model = AutoNHITS(
                h=h,
                loss=MAE(),
                config=nhits_config,
                search_alg=HyperOptSearch(),
                num_samples=10)
        
        self._model.fit(train_ts, val_size=h*2)

    
    def produce(self, *, inputs: Inputs, timeout: float = None, iterations: int = None) -> base.CallResult[Outputs]:
        print("calling produce")
        #print("Inputs:")
        #print(inputs)
        #inputs is non-target columns that can optionally be used as future exogenous data.
        #also includes time and grouping columns

        if inputs.equals(self._training_attributes):
            #D3M likes to get in-sample predictions before actual forecasts
            #neuralforecast doesn't like to do that
            #so, if inputs match training data (i.e. D3M is looking for in-sample predictions)
            #return essentially dummy data
            #hopefully this will not mess anything up down the line
            #it doesn't seem like training predictions are really used despite D3M wanting them
            #and they dont really make sense for time series forecasting
            #dataframe that is the same length as expected output
            #contains one column with the target's name which is all 0's
            print("returning dummy data for in-sample predictions")
            nrows = inputs.shape[0]
            predictions = pd.DataFrame({self._target_name:[0]*nrows})

        else:
            train_ts = TimeSeriesDataset.from_df(df=self._nf_train_data)[0]
            
            #fit if we have not fit the model yet
            #refit if there is new training data
            if not self._fitted or self._new_training_data:
                #predict for a number of periods corresponding to number of rows in inputs
                h = int(inputs.shape[0]/self._ngroups)
                #print("h:" + str(h))

                self._fit_nf(train_ts, h)

                self._fitted = True
                self._new_training_data = False #we have fit on current train data, no longer new

            #TODO: check that self._model not None
            future = self._format_data(inputs)
            print("future:")
            print(future)

            predict_ts = TimeSeriesDataset.update_dataset(
                dataset=train_ts, future_df=future
            )
            self._model.set_test_size(h)

            predictions = self._model.predict(dataset=predict_ts)
            predictions = list(predictions.flatten())
            #print("raw predictions:")
            #print(predictions)
            #print(type(predictions))
            predictions = pd.DataFrame({self._target_name : predictions})

            #print("predictions to return:")
            #print(predictions)

        #need to put predictions in right format for d3m
        output = container.DataFrame(predictions, generate_metadata=True)
        return base.CallResult(output)






import random
import math


class LoopPadding(object):
    def __init__(self, size):
        self.size = size

    def __call__(self, frame_indices):
        out = frame_indices

        for index in out:
            if len(out) >= self.size:
                break
            out.append(index)

        return out


class TemporalCenterCrop(object):
    """Temporally crop the given frame indices at a center.

    If the number of frames is less than the size,
    loop the indices as many times as necessary to satisfy the size.

    Args:
        size (int): Desired output size of the crop.
    """

    def __init__(self, size):
        self.size = size

    def __call__(self, frame_indices):
        """
        Args:
            frame_indices (list): frame indices to be cropped.
        Returns:
            list: Cropped frame indices.
        """

        center_index = len(frame_indices) // 2
        begin_index = max(0, center_index - (self.size // 2))
        end_index = min(begin_index + self.size, len(frame_indices))

        out = frame_indices[begin_index:end_index]

        for index in out:
            if len(out) >= self.size:
                break
            out.append(index)

        return out



import copy
import json
import os

import functools
import numpy as np
import torch
import torch.utils.data as data
from PIL import Image


def pil_loader(path):
    # open path as file to avoid ResourceWarning (https://github.com/python-pillow/Pillow/issues/835)
    with open(path, 'rb') as f:
        with Image.open(f) as img:
            return img.convert('RGB')


def accimage_loader(path):
    try:
        return accimage.Image(path)
    except IOError:
        # Potentially a decoding problem, fall back to PIL.Image
        return pil_loader(path)


def get_default_image_loader():
    from torchvision import get_image_backend
    if get_image_backend() == 'accimage':
        return accimage_loader
    else:
        return pil_loader


def video_loader(video_dir_path, frame_indices, image_loader):
    video = []
    for i in frame_indices:
        image_path = os.path.join(video_dir_path, 'image_{:05d}.jpg'.format(i))
        if os.path.exists(image_path):
            video.append(image_loader(image_path))
        else:
            return video

    return video


def get_default_video_loader():
    image_loader = get_default_image_loader()
    return functools.partial(video_loader, image_loader=image_loader)


def load_annotation_data(data_file_path):
    with open(data_file_path, 'r') as data_file:
        return json.load(data_file)


def get_class_labels(data):
    class_labels_map = {}
    index = 0
    for class_label in data['labels']:
        class_labels_map[class_label] = index
        index += 1
    return class_labels_map


def get_video_names_and_annotations(data, subset):
    video_names = []
    annotations = []

    for key, value in data['database'].items():
        this_subset = value['subset']
        if this_subset == subset:
            if subset == 'testing':
                video_names.append('test/{}'.format(key))
            else:
                label = value['annotations']['label']
                video_names.append('{}/{}'.format(label, key))
                annotations.append(value['annotations'])

    return video_names, annotations


def make_dataset(video_path, sample_duration, down_rate):
    dataset = []

    n_frames = len(os.listdir(video_path))

    begin_t = 1
    end_t = n_frames
    sample = {
        'video': video_path,
        'segment': [begin_t, end_t],
        'n_frames': n_frames,
    }

    step = sample_duration * down_rate
    for i in range(1, (n_frames - sample_duration + 1), step):
        sample_i = copy.deepcopy(sample)
        sample_i['frame_indices'] = list(range(i, i + sample_duration))
        sample_i['segment'] = torch.IntTensor([i, i + sample_duration - 1])
        dataset.append(sample_i)

    return dataset

def make_dataset_from_matrix(vid_matrix, sample_duration, down_rate):
    dataset = []
    n_frames = vid_matrix.shape[0]
    begin_t = 1
    end_t = n_frames
    sample = {
        'video': vid_matrix,
        'segment': [begin_t, end_t],
        'n_frames': n_frames,
    }
    step = sample_duration * down_rate
    # for i in range(1, n_frames - sample_duration + 1, step):
    #     sample_i = copy.deepcopy(sample)
    #     sample_i['frame_indices'] = list(range(i, i + sample_duration))
    #     sample_i['segment'] = torch.IntTensor([i, i + sample_duration - 1])
    #     dataset.append(sample_i)
    for j in range(1, n_frames, step):
        sample_j = copy.deepcopy(sample)
        sample_j['frame_indices'] = list(range(j, min(n_frames + 1, j + sample_duration)))
        dataset.append(sample_j)

    return dataset



class Video(data.Dataset):
    def __init__(self, vid_content,
                 spatial_transform=None, temporal_transform=None,
                 sample_duration=16, get_loader=get_default_video_loader, down_rate=1):
        if isinstance(vid_content, str):
            self.data = make_dataset(vid_content, sample_duration, down_rate)
        elif isinstance(vid_content,np.ndarray):
            self.data = make_dataset_from_matrix(vid_content, sample_duration, down_rate)
        else:
            print('do not support input to class Video with type {}'.format(type(vid_content)))

        # data is list of dicts with keys etc."frame_indicies","segments"
        self.spatial_transform = spatial_transform
        self.temporal_transform = temporal_transform
        self.loader = get_loader()

    def __getitem__(self, index):
        """
        Args:
            index (int): Index
        Returns:
            tuple: (image, target) where target is class_index of the target class.
        """
        vid_content = self.data[index]['video']

        frame_indices = self.data[index]['frame_indices']
        if self.temporal_transform is not None:
            frame_indices = self.temporal_transform(frame_indices)
        if isinstance(vid_content,str):
            clip = self.loader(vid_content, frame_indices)
        elif isinstance(vid_content,np.ndarray):
            # clip = vid_content[frame_indices, :, :, :]
            clip = [Image.fromarray(vid_content[j-1,:,:,:]) for j in frame_indices]
        # a list(len=16) of PIL images!
        # can also take numpy.ndarray (H x W x C) instead
        if self.spatial_transform is not None:
            clip = [self.spatial_transform(img) for img in clip]
        clip = torch.stack(clip, 0).permute(1, 0, 2, 3)
        # is originally 3x16x112x112
        # target = self.data[index]['n_frames']

        return clip # , target

    def __len__(self):
        return len(self.data)



import torch
from torch import nn

from autonbox.contrib.resnet.models import resnet, pre_act_resnet, wide_resnet, resnext, densenet


def generate_model(opt):
    assert opt.mode in ['score', 'feature']
    if opt.mode == 'score':
        last_fc = True
    elif opt.mode == 'feature':
        last_fc = False

    assert opt.model_name in ['resnet', 'preresnet', 'wideresnet', 'resnext', 'densenet']

    if opt.model_name == 'resnet':
        assert opt.model_depth in [10, 18, 34, 50, 101, 152, 200]

        if opt.model_depth == 10:
            model = resnet.resnet10(num_classes=opt.n_classes, shortcut_type=opt.resnet_shortcut,
                                    sample_size=opt.sample_size, sample_duration=opt.sample_duration,
                                    last_fc=last_fc)
        elif opt.model_depth == 18:
            model = resnet.resnet18(num_classes=opt.n_classes, shortcut_type=opt.resnet_shortcut,
                                    sample_size=opt.sample_size, sample_duration=opt.sample_duration,
                                    last_fc=last_fc)
        elif opt.model_depth == 34:
            model = resnet.resnet34(num_classes=opt.n_classes, shortcut_type=opt.resnet_shortcut,
                                    sample_size=opt.sample_size, sample_duration=opt.sample_duration,
                                    last_fc=last_fc)
        elif opt.model_depth == 50:
            model = resnet.resnet50(num_classes=opt.n_classes, shortcut_type=opt.resnet_shortcut,
                                    sample_size=opt.sample_size, sample_duration=opt.sample_duration,
                                    last_fc=last_fc)
        elif opt.model_depth == 101:
            model = resnet.resnet101(num_classes=opt.n_classes, shortcut_type=opt.resnet_shortcut,
                                     sample_size=opt.sample_size, sample_duration=opt.sample_duration,
                                     last_fc=last_fc)
        elif opt.model_depth == 152:
            model = resnet.resnet152(num_classes=opt.n_classes, shortcut_type=opt.resnet_shortcut,
                                     sample_size=opt.sample_size, sample_duration=opt.sample_duration,
                                     last_fc=last_fc)
        elif opt.model_depth == 200:
            model = resnet.resnet200(num_classes=opt.n_classes, shortcut_type=opt.resnet_shortcut,
                                     sample_size=opt.sample_size, sample_duration=opt.sample_duration,
                                     last_fc=last_fc)
    elif opt.model_name == 'wideresnet':
        assert opt.model_depth in [50]

        if opt.model_depth == 50:
            model = wide_resnet.resnet50(num_classes=opt.n_classes, shortcut_type=opt.resnet_shortcut, k=opt.wide_resnet_k,
                                         sample_size=opt.sample_size, sample_duration=opt.sample_duration,
                                         last_fc=last_fc)
    elif opt.model_name == 'resnext':
        assert opt.model_depth in [50, 101, 152]

        if opt.model_depth == 50:
            model = resnext.resnet50(num_classes=opt.n_classes, shortcut_type=opt.resnet_shortcut, cardinality=opt.resnext_cardinality,
                                     sample_size=opt.sample_size, sample_duration=opt.sample_duration,
                                     last_fc=last_fc)
        elif opt.model_depth == 101:
            model = resnext.resnet101(num_classes=opt.n_classes, shortcut_type=opt.resnet_shortcut, cardinality=opt.resnext_cardinality,
                                      sample_size=opt.sample_size, sample_duration=opt.sample_duration,
                                      last_fc=last_fc)
        elif opt.model_depth == 152:
            model = resnext.resnet152(num_classes=opt.n_classes, shortcut_type=opt.resnet_shortcut, cardinality=opt.resnext_cardinality,
                                      sample_size=opt.sample_size, sample_duration=opt.sample_duration,
                                      last_fc=last_fc)
    elif opt.model_name == 'preresnet':
        assert opt.model_depth in [18, 34, 50, 101, 152, 200]

        if opt.model_depth == 18:
            model = pre_act_resnet.resnet18(num_classes=opt.n_classes, shortcut_type=opt.resnet_shortcut,
                                            sample_size=opt.sample_size, sample_duration=opt.sample_duration,
                                            last_fc=last_fc)
        elif opt.model_depth == 34:
            model = pre_act_resnet.resnet34(num_classes=opt.n_classes, shortcut_type=opt.resnet_shortcut,
                                            sample_size=opt.sample_size, sample_duration=opt.sample_duration,
                                            last_fc=last_fc)
        elif opt.model_depth == 50:
            model = pre_act_resnet.resnet50(num_classes=opt.n_classes, shortcut_type=opt.resnet_shortcut,
                                            sample_size=opt.sample_size, sample_duration=opt.sample_duration,
                                            last_fc=last_fc)
        elif opt.model_depth == 101:
            model = pre_act_resnet.resnet101(num_classes=opt.n_classes, shortcut_type=opt.resnet_shortcut,
                                             sample_size=opt.sample_size, sample_duration=opt.sample_duration,
                                             last_fc=last_fc)
        elif opt.model_depth == 152:
            model = pre_act_resnet.resnet152(num_classes=opt.n_classes, shortcut_type=opt.resnet_shortcut,
                                             sample_size=opt.sample_size, sample_duration=opt.sample_duration,
                                             last_fc=last_fc)
        elif opt.model_depth == 200:
            model = pre_act_resnet.resnet200(num_classes=opt.n_classes, shortcut_type=opt.resnet_shortcut,
                                             sample_size=opt.sample_size, sample_duration=opt.sample_duration,
                                             last_fc=last_fc)
    elif opt.model_name == 'densenet':
        assert opt.model_depth in [121, 169, 201, 264]

        if opt.model_depth == 121:
            model = densenet.densenet121(num_classes=opt.n_classes,
                                         sample_size=opt.sample_size, sample_duration=opt.sample_duration,
                                         last_fc=last_fc)
        elif opt.model_depth == 169:
            model = densenet.densenet169(num_classes=opt.n_classes,
                                         sample_size=opt.sample_size, sample_duration=opt.sample_duration,
                                         last_fc=last_fc)
        elif opt.model_depth == 201:
            model = densenet.densenet201(num_classes=opt.n_classes,
                                         sample_size=opt.sample_size, sample_duration=opt.sample_duration,
                                         last_fc=last_fc)
        elif opt.model_depth == 264:
            model = densenet.densenet264(num_classes=opt.n_classes,
                                         sample_size=opt.sample_size, sample_duration=opt.sample_duration,
                                         last_fc=last_fc)

    if not opt.no_cuda:
        model = model.cuda()
        model = nn.DataParallel(model, device_ids=None)

    return model



import os
import warnings
from typing import Any, Sequence

import numpy as np

import d3m.metadata
from d3m import container, utils as d3m_utils
from d3m.metadata import base as metadata_base, hyperparams
from d3m.metadata import params
from d3m.metadata.base import PrimitiveFamily
from d3m.primitive_interfaces.supervised_learning import SupervisedLearnerPrimitiveBase
from d3m.primitive_interfaces.base import ProbabilisticCompositionalityMixin
from d3m.primitive_interfaces import base
from d3m.primitives.classification.random_forest import SKlearn as SKRandomForestClassifier
from sklearn.calibration import CalibratedClassifierCV
from sklearn.exceptions import ConvergenceWarning
from sklearn.multiclass import OneVsRestClassifier
from sklearn.utils.multiclass import type_of_target

import autonbox

Input = container.DataFrame
Output = container.DataFrame


class IterativeLabelingParams(params.Params):
    is_fitted: bool
    calibclf: Any
    prim_instance: Any


class IterativeLabelingHyperparams(hyperparams.Hyperparams):
    iters = hyperparams.UniformInt(lower=1, upper=100, default=5,
                                   semantic_types=['https://metadata.datadrivendiscovery.org/types/TuningParameter'],
                                   description='The number of iterations of labeling')
    frac = hyperparams.Uniform(lower=0.01, upper=1.0, default=0.2,
                               semantic_types=['https://metadata.datadrivendiscovery.org/types/TuningParameter'],
                               description='The fraction of unlabeled item to label')
    blackbox = hyperparams.Primitive[SupervisedLearnerPrimitiveBase](
        primitive_families=[PrimitiveFamily.CLASSIFICATION],
        default=SKRandomForestClassifier,
        semantic_types=['https://metadata.datadrivendiscovery.org/types/TuningParameter'],
        description='Black box model for the classification')
    cv = hyperparams.UniformInt(lower=1, upper=100, default=5,
                                   semantic_types=['https://metadata.datadrivendiscovery.org/types/TuningParameter'],
                                   description='The number of CV folds. Only used when the blackbox estimator '
                                               'doesn\'t suuport predict_proba()')


class IterativeLabelingPrimitive(SupervisedLearnerPrimitiveBase[Input, Output, IterativeLabelingParams, IterativeLabelingHyperparams],
				 ProbabilisticCompositionalityMixin[Input, Output, IterativeLabelingParams, IterativeLabelingHyperparams]):
    """
    Blackbox based iterative labeling for semi-supervised classification
    """

    metadata = metadata_base.PrimitiveMetadata(
        {
            'id': '6bb5824f-cf16-4615-8643-8c1758bd6751',
            'version': '0.2.1',
            "name": "Iterative labeling for semi-supervised learning",
            'description': "Blackbox based iterative labeling for semi-supervised classification",
            'python_path': 'd3m.primitives.semisupervised_classification.iterative_labeling.AutonBox',
            'source': {
                'name': autonbox.__author__,
                'uris': ['https://github.com/autonlab/autonbox'],
                'contact': 'mailto:awillia2@andrew.cmu.edu'
            },
            "installation": [{
                "type": metadata_base.PrimitiveInstallationType.PIP,
                "package": "autonbox",
                "version": "0.2.4"
            }],
            'algorithm_types': [
                metadata_base.PrimitiveAlgorithmType.ITERATIVE_LABELING,
            ],
            'primitive_family': metadata_base.PrimitiveFamily.SEMISUPERVISED_CLASSIFICATION,
        },
    )

    def __init__(self, *, hyperparams: IterativeLabelingHyperparams) -> None:
        super().__init__(hyperparams=hyperparams)
        self._prim_instance = None
        self._is_fitted = False
        self.X = None
        self.y = None
        self._iters = hyperparams['iters']
        self._frac = hyperparams['frac']
        self._cv = hyperparams['cv']

        self._calibclf = None

    def __getstate__(self):
        return (
            self.hyperparams, self._prim_instance, self._is_fitted)

    def __setstate__(self, state):
        self.hyperparams, self._prim_instance, self._is_fitted = state

    def fit(self, *, timeout: float = None, iterations: int = None) -> base.CallResult[None]:
        X = self.X
        y = self.y

        primitive = self.hyperparams['blackbox']
        primitive_hyperparams = primitive.metadata.query()['primitive_code']['class_type_arguments']['Hyperparams']
        custom_hyperparams = {'n_estimators': 100}
        if isinstance(primitive, d3m.primitive_interfaces.base.PrimitiveBaseMeta):  # is a class
            valid_params = {k: custom_hyperparams[k] for k in
                            set(custom_hyperparams).intersection(set(primitive_hyperparams.configuration))}
            self._prim_instance = primitive(
                hyperparams=primitive_hyperparams(primitive_hyperparams.defaults(), **valid_params))
        else:  # is an instance
            self._prim_instance = primitive

        # Does _prim_instance._clf support predict_proba() call?
        if not hasattr(self._prim_instance._clf, 'predict_proba'):
            calibclf = CalibratedClassifierCV(self._prim_instance._clf, cv=self._cv)
            self._calibclf = OneVsRestClassifier(calibclf)

        for labelIteration in range(self._iters):
            labeledSelector = y.iloc[:, 0].notnull() & (y.iloc[:, 0].apply(lambda x: x != ''))
            labeledIx = np.where(labeledSelector)[0]
            unlabeledIx = np.where(~labeledSelector)[0]

            if (labelIteration == 0):
                num_instances_to_label = int(self._frac * len(unlabeledIx) + 0.5)

            labeledX = X.iloc[labeledIx]
            labeledy = y.iloc[labeledIx]

            if self._calibclf is not None:
                with warnings.catch_warnings():
                    warnings.filterwarnings("ignore", category=ConvergenceWarning)
                    self._calibclf.fit(labeledX, labeledy)
                probas = self._calibclf.predict_proba(X.iloc[unlabeledIx])
            else:
                self._prim_instance.set_training_data(inputs=labeledX, outputs=labeledy)
                self._prim_instance.fit()
                probas = self._prim_instance._clf.predict_proba(X.iloc[unlabeledIx])

            entropies = np.sum(np.log2(probas.clip(0.0000001, 1.0)) * probas, axis=1)
            # join the entropies and the unlabeled indecies into a single recarray and sort it by entropy
            entIdx = np.rec.fromarrays((entropies, unlabeledIx))
            entIdx.sort(axis=0)

            labelableIndices = entIdx['f1'][-num_instances_to_label:].reshape((-1,))

            if self._calibclf is not None:
                predictions = self._calibclf.predict(X.iloc[labelableIndices])
                predictions = container.DataFrame(predictions, generate_metadata=False)
            else:
                predictions = self._prim_instance.produce(inputs=X.iloc[labelableIndices]).value
            ydf = y.iloc[labelableIndices, 0]
            ydf.loc[:] = predictions.iloc[:, 0]

        labeledSelector = y.iloc[:, 0].notnull() & (y.iloc[:, 0].apply(lambda x: x != ''))
        labeledIx = np.where(labeledSelector)[0]
        labeledX = X.iloc[labeledIx]
        labeledy = y.iloc[labeledIx]

        if self._calibclf is not None:
            with warnings.catch_warnings():
                warnings.filterwarnings("ignore", category=ConvergenceWarning)
                self._calibclf.fit(labeledX, labeledy)
        else:
            self._prim_instance.set_training_data(inputs=labeledX, outputs=labeledy)
            self._prim_instance.fit()
        self._is_fitted = True

        return base.CallResult(None)

    def set_training_data(self, *, inputs: Input, outputs: Output) -> None:
        """
         Sets input and output feature space.

        :param inputs:
        :param outputs:
        :return:
        """
        self.X = inputs
        self.y = outputs

    def get_params(self) -> IterativeLabelingParams:
        return IterativeLabelingParams(
            is_fitted=self._is_fitted,
            calibclf=self._calibclf,
            prim_instance=self._prim_instance)

    def set_params(self, *, params: IterativeLabelingParams) -> None:
        self._is_fitted = params['is_fitted']
        self._calibclf = params['calibclf']
        self._prim_instance = params['prim_instance']

    def produce(self, *, inputs: Input, timeout: float = None, iterations: int = None) -> base.CallResult[Output]:
        if self._calibclf is not None:
            pred = self._calibclf.predict(inputs)
            df = container.DataFrame(pred, generate_metadata=False)
        else:
            pred = self._prim_instance.produce(inputs=inputs)
            df = pred.value

        # if output is a binary array of floats then convert values to int
        if type_of_target(df) == 'binary' and len(df) > 0 \
                and df.iloc[0].dtype == np.float64:
            df = df.astype(int)

        output = container.DataFrame(df, generate_metadata=True)
        # if output[0].dtype == np.float64:
        #     output = output.astype(int)  # we don't want ["-1.0", "1.0"] when runtime computes the metric
        return base.CallResult(output)

    def log_likelihoods(self, *,
                        outputs: Output,
                        inputs: Input,
                        timeout: float = None,
                        iterations: int = None) -> base.CallResult[Sequence[float]]:
        results = self._prim_instance.log_likelihoods(outputs=outputs, inputs=inputs)
        return results



import os
import sys
from setuptools import setup, find_packages

PACKAGE_NAME = 'autonbox'
MINIMUM_PYTHON_VERSION = 3, 6


def check_python_version():
    """Exit when the Python version is too low."""
    if sys.version_info < MINIMUM_PYTHON_VERSION:
        sys.exit("Python {}.{}+ is required.".format(*MINIMUM_PYTHON_VERSION))


def read_package_variable(key):
    """Read the value of a variable from the package without importing."""
    module_path = os.path.join(PACKAGE_NAME, '__init__.py')
    with open(module_path) as module:
        for line in module:
            parts = line.strip().split(' ')
            if parts and parts[0] == key:
                return parts[-1].strip("'")
    raise KeyError("'{0}' not found in '{1}'".format(key, module_path))


def read_entry_points():
    with open('entry_points.ini') as entry_points:
        return entry_points.read()


check_python_version()
version = read_package_variable('__version__')

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name=PACKAGE_NAME,
    version=version,
    description='Auton Lab TA1 primitives',
    author=read_package_variable('__author__'),
    author_email=read_package_variable('__author_email__'),
    license='MIT',
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=find_packages(exclude=['docs', 'tests*']),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        'd3m',
        'Pillow>=7.1.2',
        'numpy',
        'pandas',
        'opencv-python-headless',
        'torch>=1.4.0',
        'torchvision>=0.5.0',
        'statsforecast',
        'neuralforecast>=1.3.0',
        'transformers>=4.6.0'
    ],
    entry_points=read_entry_points(),
    url='https://github.com/autonlab/autonbox',
)



import typing
from frozendict import FrozenOrderedDict

from d3m import container
from d3m.primitive_interfaces import base
from d3m.primitive_interfaces.supervised_learning import SupervisedLearnerPrimitiveBase
from d3m.metadata import base as metadata_base
from d3m.metadata import hyperparams, params
from d3m.exceptions import MissingValueError, PrimitiveNotFittedError

import autonbox
from statsforecast.arima import AutoARIMA
from autonbox import __version__

__all__ = ('AutoARIMAWrapperPrimitive',)

Inputs = container.DataFrame
Outputs = container.DataFrame

class Params(params.Params):
    fitted: bool
    new_training_data: bool
    autoARIMA: typing.Any

class Hyperparams(hyperparams.Hyperparams):

    #removing "Exogenous cols" hyperparameter because it no longer applies--just assuming all columns are exogenous
    #removing "level" hyperparam because it is not useful in a D3M pipeline
    '''
    exogenous_cols = hyperparams.List(
        elements=hyperparams.Hyperparameter[str](""),
        default=[],
        semantic_types = ["https://metadata.datadrivendiscovery.org/types/ControlParameter"],
        description = "Columns to use as exogenous variables to be passed in to AutoARIMA.fit() and AutoARIMA.predict().",
    )

    #currently, setting this to anything other than default throws an error
    level = hyperparams.List(
        elements=hyperparams.Uniform(
            default=95,
            lower=50,
            upper=100,
            lower_inclusive=True,
            upper_inclusive=False
        ),
        default=[],
        semantic_types = ["https://metadata.datadrivendiscovery.org/types/ControlParameter"],
        description="An optional list of ints between 50 and 100 representing %% confidence levels for prediction intervals",
    )
    '''

    d = hyperparams.Union(
        configuration=FrozenOrderedDict([
            ("auto",
                hyperparams.Constant(
                    default=None
                )
            ),
            ("manual", 
                hyperparams.UniformInt(
                    lower=1,
                    upper=10,
                    default=2
                )
            )
        ]),
        default="auto",
        semantic_types=["https://metadata.datadrivendiscovery.org/types/ControlParameter"],
        description="Order of first-differencing.  Either set manually, or have it be chosen automatically."
    )

    D = hyperparams.Union(
        configuration=FrozenOrderedDict([
            ("auto",
                hyperparams.Constant(
                    default=None
                )
            ),
            ("manual", 
                hyperparams.UniformInt(
                    lower=1,
                    upper=10,
                    default=2
                )
            )
        ]),
        default="auto",
        semantic_types=["https://metadata.datadrivendiscovery.org/types/ControlParameter"],
        description="Order of seasonal-differencing.  Either set manually, or have it be chosen automatically."
    )

    max_p = hyperparams.UniformInt(
        lower=1,
        upper=100,
        default=5,
        semantic_types=["https://metadata.datadrivendiscovery.org/types/ControlParameter"],
        description="Maximum value of p"
    )

    max_q = hyperparams.UniformInt(
        lower=1,
        upper=100,
        default=5,
        semantic_types=["https://metadata.datadrivendiscovery.org/types/ControlParameter"],
        description="Maximum value of q"
    )

    max_P = hyperparams.UniformInt(
        lower=1,
        upper=100,
        default=2,
        semantic_types=["https://metadata.datadrivendiscovery.org/types/ControlParameter"],
        description="Maximum value of P"
    )

    max_Q = hyperparams.UniformInt(
        lower=1,
        upper=100,
        default=2,
        semantic_types=["https://metadata.datadrivendiscovery.org/types/ControlParameter"],
        description="Maximum value of Q"
    )

    max_order = hyperparams.UniformInt(
        lower=1,
        upper=100,
        default=5,
        semantic_types=["https://metadata.datadrivendiscovery.org/types/ControlParameter"],
        description="Maximum value of p+q+P+Q if model selection is not stepwise"
    )

    max_d = hyperparams.UniformInt(
        lower=1,
        upper=10,
        default=2,
        semantic_types=["https://metadata.datadrivendiscovery.org/types/ControlParameter"],
        description="Maximum number of non-seasonal differences"
    )

    max_D = hyperparams.UniformInt(
        lower=1,
        upper=10,
        default=1,
        semantic_types=["https://metadata.datadrivendiscovery.org/types/ControlParameter"],
        description="Maximum number of seasonal differences"
    )

    start_p = hyperparams.UniformInt(
        lower=1,
        upper=10,
        default=2,
        semantic_types=["https://metadata.datadrivendiscovery.org/types/ControlParameter"],
        description="Starting value of p in stepwise procedure"
    )

    start_q = hyperparams.UniformInt(
        lower=1,
        upper=10,
        default=2,
        semantic_types=["https://metadata.datadrivendiscovery.org/types/ControlParameter"],
        description="Starting value of q in stepwise procedure"
    )

    start_P = hyperparams.UniformInt(
        lower=1,
        upper=10,
        default=1,
        semantic_types=["https://metadata.datadrivendiscovery.org/types/ControlParameter"],
        description="Starting value of P in stepwise procedure"
    )

    start_Q = hyperparams.UniformInt(
        lower=1,
        upper=10,
        default=1,
        semantic_types=["https://metadata.datadrivendiscovery.org/types/ControlParameter"],
        description="Starting value of Q in stepwise procedure"
    )

    stationary = hyperparams.UniformBool(
        default=False,
        semantic_types=["https://metadata.datadrivendiscovery.org/types/ControlParameter"],
        description="If True, restricts search to stationary models."
    )

    seasonal = hyperparams.UniformBool(
        default=True,
        semantic_types=["https://metadata.datadrivendiscovery.org/types/ControlParameter"],
        description="If False, restricts search to non-seasonal models."
    )

    ic = hyperparams.Enumeration(
        values=["aic", "aicc", "bic"],
        default="aicc",
        semantic_types=["https://metadata.datadrivendiscovery.org/types/ControlParameter"],
        description="information criterion used in model selection"
    )

    #currently changing this causes autoARIMA to fail
    stepwise = hyperparams.UniformBool(
        default=True,
        semantic_types=["https://metadata.datadrivendiscovery.org/types/ControlParameter"],
        description="If True, will do stepwise selection (faster).  Otherwise, it searches over all models.  Non-stepwise selection can be very slow, especially for seasonal models.  At the time of writing, setting stepwise to False causes AutoARIMA to fail."
    )

    nmodels=hyperparams.UniformInt(
        lower=1,
        upper=500,
        default=94,
        semantic_types=["https://metadata.datadrivendiscovery.org/types/ControlParameter"],
        description="Maximum number of models considered in the stepwise search."
    )

    trace=hyperparams.UniformBool(
        default=False,
        semantic_types=["https://metadata.datadrivendiscovery.org/types/ControlParameter"],
        description="If True, the list of ARIMA models considered will be reported."
    )

    approximation = hyperparams.Enumeration(
        values = [True, False, None],
        default=None,
        semantic_types=["https://metadata.datadrivendiscovery.org/types/ControlParameter"],
        description="If True, estimation is via conditional sums of squares and the information criteria used for model selection are approximated. The final model is still computed using maximum likelihood estimation. Approximation should be used for long time series or a high seasonal period to avoid excessive computation times.  If set to None, AutoARIMA will decide whether to approximate."
    )

    method = hyperparams.Enumeration(
        values = ("CSS", "CSS-ML", "ML", None),
        default=None,
        semantic_types=["https://metadata.datadrivendiscovery.org/types/ControlParameter"],
        description="fitting method: maximum likelihood or minimize conditional sum-of-squares.  If None, will choose automatically"
    )

    truncate = hyperparams.Union(
        configuration=FrozenOrderedDict([
            ("none",
                hyperparams.Constant(
                    default=None
                )
            ),
            ("truncate", 
                hyperparams.UniformInt(
                    lower=1,
                    upper=100,
                    default=50
                )
            )
        ]),
        default="none",
        semantic_types=["https://metadata.datadrivendiscovery.org/types/ControlParameter"],
        description="An integer value indicating how many observations to use in model selection.  The last truncate values of the series are used to select a model when truncate is not None and approximation=True. All observations are used if either truncate=None or approximation=False."
    )

    test = hyperparams.Enumeration(
        values=("kpss",),
        default="kpss",
        semantic_types=["https://metadata.datadrivendiscovery.org/types/ControlParameter"],
        description="Type of unit root test to use. See ndiffs for details.  As of this writing, this argument doesn't seem to do anything in statsforecast"
    )

    test_kwargs = hyperparams.Constant(
        default = {},
        semantic_types=["https://metadata.datadrivendiscovery.org/types/ControlParameter"],
        description = "A dictionary of keyword arguments to be passed to the unit root test.  At the time of writing, the only argument that is read is alpha."
    )

    seasonal_test = hyperparams.Enumeration(
        values=("seas", "ocsb"),
        default="seas",
        semantic_types=["https://metadata.datadrivendiscovery.org/types/ControlParameter"],
        description="This determines which method is used to select the number of seasonal differences. The default method is to use a measure of seasonal strength computed from an STL decomposition. Other possibilities involve seasonal unit root tests."
    )

    seasonal_test_kwargs = hyperparams.Constant(
        default = {},
        semantic_types=["https://metadata.datadrivendiscovery.org/types/ControlParameter"],
        description = "A dictionary of keyword arguments to be passed to the seasonal unit root test.  At the time of writing, the only argument that is read is alpha."
    )

    allowdrift = hyperparams.UniformBool(
        default=True,
        semantic_types=["https://metadata.datadrivendiscovery.org/types/ControlParameter"],
        description="If True, models with drift terms are considered."
    )

    allowmean = hyperparams.UniformBool(
        default=True,
        semantic_types=["https://metadata.datadrivendiscovery.org/types/ControlParameter"],
        description="If True, models with a non-zero mean are considered."
    )

    blambda = hyperparams.Union(
        configuration=FrozenOrderedDict([
            ("none or auto",
                hyperparams.Enumeration(
                    values = (None, "auto"),
                    default=None
                )
            ),
            ("blambda", 
                hyperparams.Uniform(
                    lower=-5.0,
                    upper=5.0,
                    default=0.0
                )
            )
        ]),
        default="none or auto",
        semantic_types=["https://metadata.datadrivendiscovery.org/types/ControlParameter"],
        description="Box-Cox transformation parameter.  If lambda=\"auto\", then a transformation is automatically selected using BoxCox.lambda. The transformation is ignored if None. Otherwise, data transformed before model is estimated."
    )

    biasadj = hyperparams.UniformBool(
        default=False,
        semantic_types=["https://metadata.datadrivendiscovery.org/types/ControlParameter"],
        description="Use adjusted back-transformed mean for Box-Cox transformations.  If transformed data is used to produce forecasts and fitted values, a regular back transformation will result in median forecasts. If biasadj is True, an adjustment will be made to produce mean forecasts and fitted values."
    )

    parallel = hyperparams.UniformBool(
        default=False,
        semantic_types=["https://metadata.datadrivendiscovery.org/types/ResourcesUseParameter"],
        description="If True and stepwise = False, then the specification search is done in parallel. This can give a significant speedup on multicore machines."
    )

    num_cores = hyperparams.Union(
        configuration=FrozenOrderedDict([
            ("none",
                hyperparams.Constant(
                    default=None
                )
            ),
            ("num_cores", 
                hyperparams.UniformInt(
                    lower=1,
                    upper=20,
                    default=2
                )
            )
        ]),
        default="num_cores",
        semantic_types=["https://metadata.datadrivendiscovery.org/types/ResourcesUseParameter"],
        description="Allows the user to specify the amount of parallel processes to be used if parallel = True and stepwise = False. If None, then the number of logical cores is automatically detected and all available cores are used."
    )

    period = hyperparams.UniformInt(
        lower=1,
        upper=1000,
        default=1,
        semantic_types=["https://metadata.datadrivendiscovery.org/types/ControlParameter"],
        description="Number of observations per unit of time. For example 24 for Hourly data."
    )

class AutoARIMAWrapperPrimitive(SupervisedLearnerPrimitiveBase[Inputs, Outputs, Params, Hyperparams]):

    """
    A wrapper primitive of statsforecast.arima.AutoARIMA
    Code Available at https://github.com/Nixtla/statsforecast/blob/main/statsforecast/arima.py#L2148-L2465

    An AutoARIMA estimator.
    Returns best ARIMA model according to either AIC, AICc or BIC value.
    The function conducts a search over possible model within the order constraints provided.

    Notes
    -----
    * This implementation is a mirror of Hyndman's forecast::auto.arima.

    References
    ----------
    [1] https://github.com/robjhyndman/forecast
    """

    metadata = metadata_base.PrimitiveMetadata({
        "id": "434d4d25-dd61-4a32-a624-0f983995e189",
        "version": "0.1.0",
        "name": "statsforecast.arima.AutoARIMA",
        "description": "Wrapper of the AutoARIMA class from statsforecast package",
        "python_path": "d3m.primitives.time_series_forecasting.arima.AutonBox",
        "primitive_family": metadata_base.PrimitiveFamily.TIME_SERIES_FORECASTING,
        "algorithm_types": ["AUTOREGRESSIVE_INTEGRATED_MOVING_AVERAGE"],
        'source': {
            'name': autonbox.__author__,
            'uris': ['https://github.com/autonlab/autonbox'],
            'contact': 'mailto:mkowales@andrew.cmu.edu'
        },
        "keywords": ["ARIMA", "time series", "forecasting"],
        "installation": [{
            "type": metadata_base.PrimitiveInstallationType.PIP,
            "package": "autonbox",
            "version": __version__
        }]
    })

    def __init__(self, *, hyperparams: Hyperparams) -> None:
        #print("calling __init__")

        super().__init__(hyperparams=hyperparams)

        self._fitted = False
        self._training_target = None
        self._training_exogenous = None
        self._new_training_data = False
        self._autoARIMA = None

    def get_params(self) -> Params:
        #print("calling get_params")
        return Params(
            fitted = self._fitted,
            new_training_data = self._new_training_data,
            autoARIMA = self._autoARIMA
        )

    def set_params(self, *, params: Params) -> None:
        #print("calling set_params, params argument:")
        #print(params)
        self._fitted = params['fitted']
        self._new_training_data = params['new_training_data']
        self._autoARIMA = params['autoARIMA']

    def set_training_data(self, *, inputs: Inputs, outputs: Outputs) -> None:
        '''
        print("calling set_training_data")
        print("Inputs:")
        print(inputs)
        print("Outputs:")
        print(outputs)
        '''
        
        '''
        inputs is a dataframe that will be used as exogenous data, excepting time columns
        outputs is a dataframe containing one column, the time series that we want to predict future values of
        '''

        #TODO: check that outputs has one column
        #TODO: check that inputs and outputs have same number of rows
        #TODO: check at np.nan and np.inf are not present

        self._training_exogenous = inputs
        self._training_target = outputs
        self._new_training_data = True

    #private method
    #determine columns to be used as exogenous data from column semantic types
    def _format_exogenous(self, inputs):
        timestamp_cols = inputs.metadata.list_columns_with_semantic_types(
            (
                "https://metadata.datadrivendiscovery.org/types/Time",
            )
        )
        #print("timestamp cols: " + str(timestamp_cols))
        #TODO: raise error if there are multiple time cols or it is not a valid time series?

        grouping_cols = inputs.metadata.list_columns_with_semantic_types(
            (
                "https://metadata.datadrivendiscovery.org/types/GroupingKey",
                "https://metadata.datadrivendiscovery.org/types/SuggestedGroupingKey"
            )
        )
        #print("grouping cols: " + str(grouping_cols))
        #TODO: raise error if there are any grouping cols

        attribute_cols = inputs.metadata.list_columns_with_semantic_types(
            (
                "https://metadata.datadrivendiscovery.org/types/Attribute",
            )
        )
        #print("attribute cols: " + str(attribute_cols))
        
        exogenous_cols = list(set(attribute_cols) - set(grouping_cols + timestamp_cols))
        #print("exogneous_cols: " + str(exogenous_cols))

        #return None
        if (exogenous_cols == []):
            return None
        else:
            exogenous = inputs.iloc[:, exogenous_cols]
            #print("exogenous: ")
            #print(exogenous)
            X = exogenous.to_numpy().astype(float)
            return X 

    def fit(self, *, timeout: float = None, iterations: int = None) -> base.CallResult[None]:
        #print("Fitting StatsForecast AutoARIMA")

        #make hyperparams into local variables for convenience
        d = self.hyperparams['d']
        D = self.hyperparams['D']
        max_p = self.hyperparams['max_p']
        max_q = self.hyperparams['max_q']
        max_P = self.hyperparams['max_P']
        max_Q = self.hyperparams['max_Q']
        max_order = self.hyperparams['max_order']
        max_d = self.hyperparams['max_d']
        max_D = self.hyperparams['max_D']
        start_p = self.hyperparams['start_p']
        start_q = self.hyperparams['start_q']
        start_P = self.hyperparams['start_P']
        start_Q = self.hyperparams['start_Q']
        stationary = self.hyperparams['stationary']
        seasonal = self.hyperparams['seasonal']
        ic = self.hyperparams['ic']
        stepwise = self.hyperparams['stepwise']
        nmodels = self.hyperparams['nmodels']
        trace = self.hyperparams['trace']
        approximation = self.hyperparams['approximation']
        method = self.hyperparams['method']
        truncate = self.hyperparams['truncate']
        test = self.hyperparams['test']
        test_kwargs = self.hyperparams['test_kwargs']
        seasonal_test = self.hyperparams['seasonal_test']
        seasonal_test_kwargs = self.hyperparams['seasonal_test_kwargs']
        allowdrift = self.hyperparams['allowdrift']
        allowmean = self.hyperparams['allowmean']
        blambda = self.hyperparams['blambda']
        biasadj = self.hyperparams['biasadj']
        parallel = self.hyperparams['parallel']
        num_cores = self.hyperparams['num_cores']
        period = self.hyperparams['period']

        self._autoARIMA = AutoARIMA(
            d = d,
            D = D,
            max_p = max_p,
            max_q = max_q,
            max_P = max_P,
            max_Q = max_Q,
            max_order = max_order,
            max_d = max_d,
            max_D = max_D,
            start_p = start_p,
            start_q = start_q,
            start_P = start_P,
            start_Q = start_Q,
            stationary = stationary,
            seasonal = seasonal,
            ic = ic,
            stepwise = stepwise,
            nmodels = nmodels,
            trace = trace,
            approximation = approximation,
            method = method,
            truncate = truncate,
            test = test,
            test_kwargs = test_kwargs,
            seasonal_test = seasonal_test,
            seasonal_test_kwargs = seasonal_test_kwargs,
            allowdrift = allowdrift,
            allowmean = allowmean,
            blambda = blambda,
            biasadj = biasadj,
            parallel = parallel,
            num_cores = num_cores,
            period = period
        )

        if self._training_target is None:
            raise MissingValueError("fit() called before training data set, call set_training_data() first.")
        
        if self._fitted == True and self._new_training_data == False:
            self.logger.warning("Model is already fit and training data has not changed.  Model will be refit from scratch, but expect nothing to change.")

        #AutoARIMA takes a 1-dimensional ndarray for y
        y = self._training_target.to_numpy().flatten()
        #print("y")
        #print(y)

        #extract exogenous columns if there are any, and turn them into a 2d numpy array of floats.
        X = self._format_exogenous(self._training_exogenous)
        #print("X:")
        #print(X)

        self._autoARIMA.fit(y=y, X=X)
        self._fitted = True
        return base.CallResult(None)

    def produce(self, *, inputs: Inputs, timeout: float = None, iterations: int = None) -> base.CallResult[Outputs]:
        #print("calling produce")
        #print("Inputs:")
        #print(inputs)

        #inputs is non-target columns that can optionally be used as future exogenous data.

        if not self._fitted:
            raise PrimitiveNotFittedError("Primitive not fitted.")

        #predict for a number of periods corresponding to number of rows in inputs
        nrows = inputs.shape[0]
        #print("nrows:")
        #print(nrows)
        
        X = self._format_exogenous(inputs)
        #print("X:")
        #print(X)

        predictions = self._autoARIMA.predict(h=nrows, X=X, level=[])
        #print("predictions:")
        #print(predictions)
        output = container.DataFrame(predictions, generate_metadata=True)
        return base.CallResult(output)






import unittest
import os

import numpy as np
import pandas as pd
from ray import tune
from ray.tune.search.hyperopt import HyperOptSearch
#from neuralforecast import NeuralForecast
from neuralforecast.tsdataset import TimeSeriesDataset
from neuralforecast.auto import AutoNHITS
from neuralforecast.losses.pytorch import MAE

from d3m import runtime, index
from d3m.container import dataset
from d3m.metadata import problem
from d3m.metadata.base import ArgumentType, Context
from d3m.metadata.pipeline import Pipeline, PrimitiveStep

class AutoNHITSTestCase(unittest.TestCase):

    #hyperparams argument is a list of (name, data) tuples
    def construct_pipeline(self, hyperparams) -> Pipeline:
        # Creating pipeline
        pipeline_description = Pipeline()
        pipeline_description.add_input(name='inputs')

        # Step 0: dataset_to_dataframe
        step_0 = PrimitiveStep(primitive=index.get_primitive('d3m.primitives.data_transformation.dataset_to_dataframe.Common'))
        step_0.add_argument(name='inputs', argument_type=ArgumentType.CONTAINER, data='inputs.0')
        step_0.add_output('produce')
        pipeline_description.add_step(step_0)

        # Step 1: profiler
        # Automatically determine semantic types of columns
        step_1 = PrimitiveStep(primitive=index.get_primitive('d3m.primitives.schema_discovery.profiler.Common'))
        step_1.add_argument(name='inputs', argument_type=ArgumentType.CONTAINER, data='steps.0.produce')
        step_1.add_output('produce')
        pipeline_description.add_step(step_1)

        # Step 2: column parser
        step_2 = PrimitiveStep(primitive=index.get_primitive('d3m.primitives.data_transformation.column_parser.Common'))
        step_2.add_argument(name='inputs', argument_type=ArgumentType.CONTAINER, data='steps.1.produce')
        step_2.add_output('produce')

        #not adding this hyperparameter messes it up
        #I have NO IDEA why
        #looking at the documentation for this primitive,
        #  using defaults should be better
        #  if if you use defaults it messes up the values in a lot of the columns
        step_2.add_hyperparameter(
            name='parse_semantic_types',
            argument_type=ArgumentType.VALUE,
            data=[
                "http://schema.org/Boolean",
                "http://schema.org/Integer",
                "http://schema.org/Float",
                "https://metadata.datadrivendiscovery.org/types/FloatVector"
            ]
        )

        pipeline_description.add_step(step_2)

        # Step 3: extract_columns_by_semantic_types(attributes)
        step_3 = PrimitiveStep(primitive=index.get_primitive('d3m.primitives.data_transformation.extract_columns_by_semantic_types.Common'))
        step_3.add_argument(name='inputs', argument_type=ArgumentType.CONTAINER, data='steps.2.produce')
        step_3.add_output('produce')
        step_3.add_hyperparameter(
            name='semantic_types',
            argument_type=ArgumentType.VALUE,
            data=['https://metadata.datadrivendiscovery.org/types/Attribute'],
        )
        pipeline_description.add_step(step_3)

        # Step 4: extract_columns_by_semantic_types(targets)
        step_4 = PrimitiveStep(primitive=index.get_primitive('d3m.primitives.data_transformation.extract_columns_by_semantic_types.Common'))
        step_4.add_argument(name='inputs', argument_type=ArgumentType.CONTAINER, data='steps.2.produce')
        step_4.add_output('produce')
        step_4.add_hyperparameter(
            name='semantic_types',
            argument_type=ArgumentType.VALUE,
            data=[
                "https://metadata.datadrivendiscovery.org/types/Target",
                "https://metadata.datadrivendiscovery.org/types/TrueTarget",
                "https://metadata.datadrivendiscovery.org/types/SuggestedTarget"
            ]
        )
        pipeline_description.add_step(step_4)

        # Step 5: autoNHITS
        step_5 = PrimitiveStep(primitive=index.get_primitive('d3m.primitives.time_series_forecasting.nhits.AutonBox'))
        step_5.add_argument(name='inputs', argument_type=ArgumentType.CONTAINER, data='steps.3.produce')
        step_5.add_argument(name='outputs', argument_type=ArgumentType.CONTAINER, data='steps.4.produce')
        #add hyperparams from argument
        for h in hyperparams:
            (name, data) = h
            step_5.add_hyperparameter(
                name = name,
                argument_type = ArgumentType.VALUE,
                data = data
            )
        step_5.add_output('produce')
        pipeline_description.add_step(step_5)

        # Step 6: construct_predictions
        # This is a primitive which assures that the output of a standard pipeline has predictions
        # in the correct structure (e.g., there is also a d3mIndex column with index for every row).
        step_6 = PrimitiveStep(primitive=index.get_primitive('d3m.primitives.data_transformation.construct_predictions.Common'))
        step_6.add_argument(name='inputs', argument_type=ArgumentType.CONTAINER, data='steps.5.produce')
        # This is a primitive which uses a non-standard second argument, named "reference".
        step_6.add_argument(name='reference', argument_type=ArgumentType.CONTAINER, data='steps.0.produce')
        step_6.add_output('produce')
        pipeline_description.add_step(step_6)

        # Final output
        pipeline_description.add_output(name='output predictions', data_reference='steps.6.produce')

        # print json for reference
        #print(pipeline_description.to_json())

        return pipeline_description

    def run_pipeline(self, pipeline_description : Pipeline, dataset_location : str):
        problem_path = os.path.join(dataset_location, "TRAIN", "problem_TRAIN", "problemDoc.json")
        train_doc_path = os.path.join(dataset_location, "TRAIN", "dataset_TRAIN", "datasetDoc.json")
        test_doc_path = os.path.join(dataset_location, "TEST", "dataset_TEST", "datasetDoc.json")

        # Loading problem description.
        problem_description = problem.get_problem(problem_path)

        # Loading train and test datasets.
        train_dataset = dataset.get_dataset(train_doc_path)
        test_dataset = dataset.get_dataset(test_doc_path)

        print(train_dataset)
        print(test_dataset)

        # Fitting pipeline on train dataset.
        fitted_pipeline, train_predictions, fit_result = runtime.fit(
            pipeline_description,
            [train_dataset],
            problem_description=problem_description,
            context=Context.TESTING,
        )

        # Any errors from running the pipeline are captured and stored in
        # the result objects (together with any values produced until then and
        # pipeline run information). Here we just want to know if it succeed.
        fit_result.check_success()

        # Producing predictions using the fitted pipeline on test dataset.
        test_predictions, produce_result = runtime.produce(
            fitted_pipeline,
            [test_dataset],
        )
        produce_result.check_success()

        return test_predictions
    
    def run_direct(self, train, test, h):
        #run AutoNHITS directly
        future_exog = list(set(train.columns) - set(['ds', 'unique_id', 'y']))
        
        print("Fitting NeuralForecast AutoNHITS (direct)")
        print("train:")
        print(train)
        print("h:" + str(h))
        print("future exog: " + str(future_exog))

        nhits_config = {
            "input_size": 3*h,
            "n_pool_kernel_size": tune.choice(
                [3 * [1], 3 * [2], 3 * [4], [8, 4, 1], [16, 8, 1]]
            ),
            "n_freq_downsample": tune.choice(
                [
                    [168, 24, 1],
                    [24, 12, 1],
                    [180, 60, 1],
                    [60, 8, 1],
                    [40, 20, 1],
                    [1, 1, 1],
                ]
            ),
            "learning_rate": tune.loguniform(1e-4, 1e-1),
            "scaler_type" : 'robust',
            "max_steps": 100,  #TODO: change to 1000 after testing
            "batch_size": tune.choice([32, 64, 128, 256]),
            "windows_batch_size": tune.choice([128, 256, 512, 1024]),                                                                       # Initial Learning rate
            "random_seed": 1,                               
            "futr_exog_list" : future_exog
        }
        
        model = AutoNHITS(
                h=h,
                loss=MAE(),
                config=nhits_config,
                search_alg=HyperOptSearch(),
                num_samples=10)

        train, uids, last_dates, ds = TimeSeriesDataset.from_df(df=train)
        model.fit(train, val_size=h*2)

        y = test['y']
        del test['y']

        print("future:")
        print(test)
        dataset = TimeSeriesDataset.update_dataset(
            dataset=train, future_df=test
        )

        model.set_test_size(h)  # To predict h steps ahead
        model_fcsts = model.predict(dataset=dataset)
        #print("model_fcsts:")
        #print(model_fcsts)
        return(pd.DataFrame({"y": list(model_fcsts.flatten())}))
        #----------------------------------
        '''
        if issubclass(last_dates.dtype.type, np.integer):
            last_date_f = lambda x: np.arange(
            	x + 1, x + 1 + h, dtype=last_dates.dtype
        	)
        else:
            last_date_f = lambda x: pd.date_range(
                x + self.freq, periods=h, freq=self.freq
            )

        if len(np.unique(last_dates)) == 1:
            dates = np.tile(last_date_f(last_dates[0]), len(train))
        else:
            dates = np.hstack([last_date_f(last_date) for last_date in last_dates])
        
        idx = pd.Index(np.repeat(uids, h), name="unique_id")
        fcsts_df  = pd.DataFrame({"ds": dates}, index=idx)

        

        col_idx = 0
        fcsts = np.full((h * len(uids), 1), fill_value=np.nan)
        

        # Append predictions in memory placeholder
        output_length = len(model.loss.output_names)
        fcsts[:, col_idx : col_idx + output_length] = model_fcsts
        col_idx += output_length

        # Declare predictions pd.DataFrame
        fcsts = pd.DataFrame.from_records(fcsts, columns=cols, index=fcsts_df.index)
        fcsts_df = pd.concat([fcsts_df, fcsts], axis=1)
        '''
        #----------------------------------

    def test_nfsample(self):
        print("testing nf sample dataset")
        dataset_location = "/home/mkowales/datasets/nfsample/d3m"

        train_data_path = os.path.join(dataset_location, "TRAIN", "dataset_TRAIN", "tables", "learningData.csv")
        test_data_path = os.path.join(dataset_location, "TEST", "dataset_TEST", "tables", "learningData.csv")

        target_name = 'y'

        train = pd.read_csv(train_data_path)
        test = pd.read_csv(test_data_path)
        train['ds'] = pd.to_datetime(train['ds'])
        test['ds'] = pd.to_datetime(test['ds'])

        del train['d3mIndex']
        del test['d3mIndex']

        h = int(test.shape[0]/2)

        #----------

        #run AutoNHITS directly
        direct_predictions = self.run_direct(train, test, h)

        #run simple pipeline with AutoNHITS primitive
        pipeline_description = self.construct_pipeline(hyperparams=[])
        pipeline_predictions = self.run_pipeline(pipeline_description, dataset_location)
        pipeline_predictions = pipeline_predictions[target_name]

        print("direct:")
        print(direct_predictions)
        print(type(direct_predictions))
        print("from pipeline:")
        print(pipeline_predictions)
        print(type(pipeline_predictions))

        #predictions will not necessarily be identical but should be similar
        #assert((direct_predictions['y'] == pipeline_predictions).all())

    def test_sunspots(self):

        dataset_location = "/home/mkowales/datasets/sunspots/d3m"

        train_data_path = os.path.join(dataset_location, "TRAIN", "dataset_TRAIN", "tables", "learningData.csv")
        test_data_path = os.path.join(dataset_location, "TEST", "dataset_TEST", "tables", "learningData.csv")

        target_name = 'sunspots'

        train = pd.read_csv(train_data_path)
        test = pd.read_csv(test_data_path)
        train['ds'] = pd.to_datetime(train['year'], format="%Y")
        test['ds'] = pd.to_datetime(test['year'], format="%Y")
        del train['year']
        del test['year']

        train['unique_id'] = ['a']*train.shape[0]
        test['unique_id'] = ['a']*test.shape[0]

        train.rename(columns={"sunspots":"y"}, inplace=True)
        test.rename(columns={"sunspots":"y"}, inplace=True)

        h = int(test.shape[0])

        #----------
        #run AutoNHITS directly
        direct_predictions = self.run_direct(train, test, h)

        #run simple pipeline with AutoNHITS primitive
        pipeline_description = self.construct_pipeline(hyperparams=[])
        pipeline_predictions = self.run_pipeline(pipeline_description, dataset_location)
        pipeline_predictions = pipeline_predictions[target_name]

        print("direct:")
        print(direct_predictions)
        print("from pipeline:")
        print(pipeline_predictions)

        #predictions will not necessarily be identical but should be similar
        #assert((direct_predictions == pipeline_predictions).all())
    
if __name__ == '__main__':
    unittest.main()